==> retro_data_structures-0.28.0-py3-none-any.whl: retro_data_structures/properties/corruption/archetypes/ScaleSplines.py <==
# Generated File
from __future__ import annotations

import dataclasses
import struct
import typing
import typing_extensions

from retro_data_structures import json_util
from retro_data_structures.game_check import Game
from retro_data_structures.properties.base_property import BaseProperty
from retro_data_structures.properties.field_reflection import FieldReflection
from retro_data_structures.properties.corruption.core.Spline import Spline

if typing.TYPE_CHECKING:
    class ScaleSplinesJson(typing_extensions.TypedDict):
        x_scale: json_util.JsonObject
        y_scale: json_util.JsonObject
        z_scale: json_util.JsonObject
    

@dataclasses.dataclass()
class ScaleSplines(BaseProperty):
    x_scale: Spline = dataclasses.field(default_factory=Spline, metadata={
        'reflection': FieldReflection[Spline](
            Spline, id=0xf437a62f, original_name='XScale', from_json=Spline.from_json, to_json=Spline.to_json
        ),
    })
    y_scale: Spline = dataclasses.field(default_factory=Spline, metadata={
        'reflection': FieldReflection[Spline](
            Spline, id=0x6f92ea40, original_name='YScale', from_json=Spline.from_json, to_json=Spline.to_json
        ),
    })
    z_scale: Spline = dataclasses.field(default_factory=Spline, metadata={
        'reflection': FieldReflection[Spline](
            Spline, id=0x180c38b0, original_name='ZScale', from_json=Spline.from_json, to_json=Spline.to_json
        ),
    })

    @classmethod
    def game(cls) -> Game:
        return Game.CORRUPTION

    @classmethod
    def from_stream(cls, data: typing.BinaryIO, size: int | None = None, default_override: dict | None = None) -> typing_extensions.Self:
        property_count = struct.unpack(">H", data.read(2))[0]
        if (result := cls._fast_decode(data, property_count)) is not None:
            return result

        present_fields = default_override or {}
        for _ in range(property_count):
            property_id, property_size = struct.unpack(">LH", data.read(6))
            start = data.tell()
            try:
                property_name, decoder = _property_decoder[property_id]
                present_fields[property_name] = decoder(data, property_size)
            except KeyError:
                raise RuntimeError(f"Unknown property: 0x{property_id:08x}")
            assert data.tell() - start == property_size

        return cls(**present_fields)

    @classmethod
    def _fast_decode(cls, data: typing.BinaryIO, property_count: int) -> typing_extensions.Self | None:
        if property_count != 3:
            return None
    
        property_id, property_size = struct.unpack(">LH", data.read(6))
        assert property_id == 0xf437a62f
        x_scale = Spline.from_stream(data, property_size)
    
        property_id, property_size = struct.unpack(">LH", data.read(6))
        assert property_id == 0x6f92ea40
        y_scale = Spline.from_stream(data, property_size)
    
        property_id, property_size = struct.unpack(">LH", data.read(6))
        assert property_id == 0x180c38b0
        z_scale = Spline.from_stream(data, property_size)
    
        return cls(x_scale, y_scale, z_scale)

    def to_stream(self, data: typing.BinaryIO, default_override: dict | None = None) -> None:
        default_override = default_override or {}
        data.write(b'\x00\x03')  # 3 properties

        data.write(b'\xf47\xa6/')  # 0xf437a62f
        before = data.tell()
        data.write(b'\x00\x00')  # size placeholder
        self.x_scale.to_stream(data)
        after = data.tell()
        data.seek(before)
        data.write(struct.pack(">H", after - before - 2))
        data.seek(after)

        data.write(b'o\x92\xea@')  # 0x6f92ea40
        before = data.tell()
        data.write(b'\x00\x00')  # size placeholder
        self.y_scale.to_stream(data)
        after = data.tell()
        data.seek(before)
        data.write(struct.pack(">H", after - before - 2))
        data.seek(after)

        data.write(b'\x18\x0c8\xb0')  # 0x180c38b0
        before = data.tell()
        data.write(b'\x00\x00')  # size placeholder
        self.z_scale.to_stream(data)
        after = data.tell()
        data.seek(before)
        data.write(struct.pack(">H", after - before - 2))
        data.seek(after)

    @classmethod
    def from_json(cls, data: json_util.JsonValue) -> typing_extensions.Self:
        json_data = typing.cast("ScaleSplinesJson", data)
        return cls(
            x_scale=Spline.from_json(json_data['x_scale']),
            y_scale=Spline.from_json(json_data['y_scale']),
            z_scale=Spline.from_json(json_data['z_scale']),
        )

    def to_json(self) -> json_util.JsonObject:
        return {
            'x_scale': self.x_scale.to_json(),
            'y_scale': self.y_scale.to_json(),
            'z_scale': self.z_scale.to_json(),
        }


_property_decoder: typing.Dict[int, typing.Tuple[str, typing.Callable[[typing.BinaryIO, int], typing.Any]]] = {
    0xf437a62f: ('x_scale', Spline.from_stream),
    0x6f92ea40: ('y_scale', Spline.from_stream),
    0x180c38b0: ('z_scale', Spline.from_stream),
}

==> retro_data_structures-0.29.0-py3-none-any.whl: retro_data_structures/properties/corruption/archetypes/ScaleSplines.py <==
# Generated File
from __future__ import annotations

import dataclasses
import struct
import typing
import typing_extensions

from retro_data_structures import json_util
from retro_data_structures.game_check import Game
from retro_data_structures.properties.base_property import BaseProperty
from retro_data_structures.properties.field_reflection import FieldReflection
from retro_data_structures.properties.corruption.core.Spline import Spline

if typing.TYPE_CHECKING:
    class ScaleSplinesJson(typing_extensions.TypedDict):
        x_scale: json_util.JsonObject
        y_scale: json_util.JsonObject
        z_scale: json_util.JsonObject
    

@dataclasses.dataclass()
class ScaleSplines(BaseProperty):
    x_scale: Spline = dataclasses.field(default_factory=Spline, metadata={
        'reflection': FieldReflection[Spline](
            Spline, id=0xf437a62f, original_name='XScale', from_json=Spline.from_json, to_json=Spline.to_json
        ),
    })
    y_scale: Spline = dataclasses.field(default_factory=Spline, metadata={
        'reflection': FieldReflection[Spline](
            Spline, id=0x6f92ea40, original_name='YScale', from_json=Spline.from_json, to_json=Spline.to_json
        ),
    })
    z_scale: Spline = dataclasses.field(default_factory=Spline, metadata={
        'reflection': FieldReflection[Spline](
            Spline, id=0x180c38b0, original_name='ZScale', from_json=Spline.from_json, to_json=Spline.to_json
        ),
    })

    @classmethod
    def game(cls) -> Game:
        return Game.CORRUPTION

    @classmethod
    def from_stream(cls, data: typing.BinaryIO, size: int | None = None, default_override: dict | None = None) -> typing_extensions.Self:
        property_count = struct.unpack(">H", data.read(2))[0]
        if (result := cls._fast_decode(data, property_count)) is not None:
            return result

        present_fields = default_override or {}
        for _ in range(property_count):
            property_id, property_size = struct.unpack(">LH", data.read(6))
            start = data.tell()
            try:
                property_name, decoder = _property_decoder[property_id]
                present_fields[property_name] = decoder(data, property_size)
            except KeyError:
                raise RuntimeError(f"Unknown property: 0x{property_id:08x}")
            assert data.tell() - start == property_size

        return cls(**present_fields)

    @classmethod
    def _fast_decode(cls, data: typing.BinaryIO, property_count: int) -> typing_extensions.Self | None:
        if property_count != 3:
            return None
    
        property_id, property_size = struct.unpack(">LH", data.read(6))
        assert property_id == 0xf437a62f
        x_scale = Spline.from_stream(data, property_size)
    
        property_id, property_size = struct.unpack(">LH", data.read(6))
        assert property_id == 0x6f92ea40
        y_scale = Spline.from_stream(data, property_size)
    
        property_id, property_size = struct.unpack(">LH", data.read(6))
        assert property_id == 0x180c38b0
        z_scale = Spline.from_stream(data, property_size)
    
        return cls(x_scale, y_scale, z_scale)

    def to_stream(self, data: typing.BinaryIO, default_override: dict | None = None) -> None:
        default_override = default_override or {}
        data.write(b'\x00\x03')  # 3 properties

        data.write(b'\xf47\xa6/')  # 0xf437a62f
        before = data.tell()
        data.write(b'\x00\x00')  # size placeholder
        self.x_scale.to_stream(data)
        after = data.tell()
        data.seek(before)
        data.write(struct.pack(">H", after - before - 2))
        data.seek(after)

        data.write(b'o\x92\xea@')  # 0x6f92ea40
        before = data.tell()
        data.write(b'\x00\x00')  # size placeholder
        self.y_scale.to_stream(data)
        after = data.tell()
        data.seek(before)
        data.write(struct.pack(">H", after - before - 2))
        data.seek(after)

        data.write(b'\x18\x0c8\xb0')  # 0x180c38b0
        before = data.tell()
        data.write(b'\x00\x00')  # size placeholder
        self.z_scale.to_stream(data)
        after = data.tell()
        data.seek(before)
        data.write(struct.pack(">H", after - before - 2))
        data.seek(after)

    @classmethod
    def from_json(cls, data: json_util.JsonValue) -> typing_extensions.Self:
        json_data = typing.cast("ScaleSplinesJson", data)
        return cls(
            x_scale=Spline.from_json(json_data['x_scale']),
            y_scale=Spline.from_json(json_data['y_scale']),
            z_scale=Spline.from_json(json_data['z_scale']),
        )

    def to_json(self) -> json_util.JsonObject:
        return {
            'x_scale': self.x_scale.to_json(),
            'y_scale': self.y_scale.to_json(),
            'z_scale': self.z_scale.to_json(),
        }


_property_decoder: dict[int, tuple[str, typing.Callable[[typing.BinaryIO, int], typing.Any]]] = {
    0xf437a62f: ('x_scale', Spline.from_stream),
    0x6f92ea40: ('y_scale', Spline.from_stream),
    0x180c38b0: ('z_scale', Spline.from_stream),
}
